@@ -2,7 +2,6 @@ from neomodel import (
     FloatProperty,
     IntegerProperty,
     Relationship,
-    RelationshipTo,
     StringProperty,
     StructuredNode,
     StructuredRel,
@@ -14,16 +13,19 @@ from neomodel import (
 
 class Client(StructuredNode):
     client_id = StringProperty(required=True, unique_index = True)
-    client_taxonomy_config = StringProperty(required=True)
+    client_name = StringProperty(required=True)
+    client_taxonomy_config = StringProperty(required=False)
 
 
 class ProblemSolution(StructuredNode):
+    #client_id = StringProperty(required=True, index=True)
     argument_type = StringProperty(required=True)  # Problem or Solution
     pro_con_flag = StringProperty(required=True)  # Pro or Con
     process_datetime = DateTimeProperty(default_now=True)
 
 
 class Topic(StructuredNode):
+    #client_id = StringProperty(required=True, index=True)
     text = StringProperty(required=True, unique_index = True)
     tsne_coords = StringProperty(required=True)
     process_datetime = DateTimeProperty(default_now=True)
@@ -33,6 +35,7 @@ class TopicRel(StructuredRel):
 
 
 class SubTopic(StructuredNode):
+    #client_id = StringProperty(required=True, index=True)
     text = StringProperty(required=True, unique_index = True)
     topic = Relationship(Topic, 'HAS_TOPIC', model=TopicRel)
     process_datetime = DateTimeProperty(default_now=True)
@@ -42,16 +45,19 @@ class SubTopicRel(StructuredRel):
 
 
 class Sentiment(StructuredNode):
+    #client_id = StringProperty(required=True, index=True)
     sentiment = IntegerProperty(required=True, unique_index=True)
     process_datetime = DateTimeProperty(default_now=True)
 
 
 class UserMood(StructuredNode):
+    #client_id = StringProperty(required=True, index=True)
     mood = IntegerProperty(required=True, unique_index=True)
 
     
 class Emoji(StructuredNode):
-    label = StringProperty(required=True)
+    #client_id = StringProperty(required=True, index=True)
+    label = StringProperty(required=True, unique_index=True)
     pax_label = StringProperty(required=True)
     group = StringProperty(required=True)
     plutchik_category = StringProperty()
@@ -64,13 +70,15 @@ class EmojiRel(StructuredRel):
 
 
 class EntityType(StructuredNode):
+    #client_id = StringProperty(required=True, index=True)
     text = StringProperty(required=True, unique_index = True)
     process_datetime = DateTimeProperty(default_now=True)
 
 
 class Entity(StructuredNode):
+    client_id = StringProperty(required=True, index=True)
     uid = UniqueIdProperty()
-    name = StringProperty(required=True)
+    name = StringProperty(required=True, unique_index = True)
     type = Relationship(EntityType, 'OF_TYPE')
     process_datetime = DateTimeProperty(default_now=True)
     
@@ -80,7 +88,7 @@ class EntityTypeRel(StructuredRel):
 
 
 class UserDemographics(StructuredNode):
-    client_id = StringProperty(required=True)
+    client_id = StringProperty(required=True, index=True)
     taxonomy = StringProperty(required=True)
     value = StringProperty(required=True)
     category = StringProperty()
@@ -117,13 +125,13 @@ class Occupation(UserDemographics):
 
 class UserCommunicationRel(StructuredRel):
     number_of_comunications=IntegerProperty(default=0)
-    comunication_start_date=DateTimeProperty(default_now=True)
+    first_comunication_date=DateTimeProperty(default_now=True)
     last_comunication_date=DateTimeProperty(default_now=True)
     
 
 class User(StructuredNode):
     uid = StringProperty(required=True, unique_index = True)
-    client_id = StringProperty(required=True)
+    client_id = StringProperty(required=True, index=True)
     anonymous = BooleanProperty()
     process_datetime = DateTimeProperty(default_now=True)
     communicates_with = Relationship('User', 'COMMUNICATES_WITH', model=UserCommunicationRel)
@@ -142,12 +150,17 @@ class User(StructuredNode):
                 'Ethnicity':'ethnicity',
             }
     
+
+class UserGroup(User):
+    members = Relationship(User, 'HAS_MEMBER')
+    
 class Comment(StructuredNode):
     uid = StringProperty(required=True, unique_index = True)
     text = StringProperty(required=True)
-    client_id = StringProperty(required=True)
-    forum_id = StringProperty(required=True)
+    client_id = StringProperty(required=True, index=True)
+    forum_id = StringProperty(required=True, index=True)
     conversation_id = StringProperty()
+    conversation_text = StringProperty()
     create_datetime = DateTimeProperty(default_now=False)
     process_datetime = DateTimeProperty(default_now=True)
     number_likes = IntegerProperty(default=0)
@@ -161,12 +174,16 @@ class Comment(StructuredNode):
     user = Relationship(User, 'POSTED_BY')
     entity = Relationship(Entity, 'HAS_ENTITY')
     user_mood = Relationship(UserMood, 'HAS_USER_MOOD')
+    argument = Relationship(ProblemSolution, 'HAS_ARGUMENT')
+    conversation = Relationship(ProblemSolution, 'HAS_CONVERSATION')
+    parent_thread_id = StringProperty(required=False)
+    parent_thread = Relationship('Comment', 'HAS_PARENT_THREAD')
 
 class Sentence(StructuredNode):
     uid = UniqueIdProperty()
     text = StringProperty(required=True)
-    client_id = StringProperty(required=True)
-    forum_id = StringProperty(required=True)
+    client_id = StringProperty(required=True, index=True)
+    forum_id = StringProperty(required=True, index=True)
     conversation_id = StringProperty()
     process_datetime = DateTimeProperty(default_now=True)
     comment = Relationship(Comment, 'DECOMPOSES')
@@ -179,8 +196,8 @@ class Sentence(StructuredNode):
 class Chunk(StructuredNode):
     uid = UniqueIdProperty()
     text = StringProperty(required=True)
-    client_id = StringProperty(required=True)
-    forum_id = StringProperty(required=True)
+    client_id = StringProperty(required=True, index=True)
+    forum_id = StringProperty(required=True, index=True)
     conversation_id = StringProperty()
     process_datetime = DateTimeProperty(default_now=True)
     comment = Relationship(Comment, 'DECOMPOSES')
@@ -192,11 +209,20 @@ class Chunk(StructuredNode):
     argument = Relationship(ProblemSolution, 'HAS_ARGUMENT')
 
 
+class Conversation(StructuredNode):
+    uid = StringProperty(required=True, unique_index = True)
+    client_id = StringProperty(required=True, index=True)
+    forum_id = StringProperty(required=True, index=True)
+    text = StringProperty(required=True)
+    description = StringProperty(required=False)
+    process_datetime = DateTimeProperty(default_now=True)
+
+    
 class Question(StructuredNode):
     uid = UniqueIdProperty()
     text = StringProperty(required=True)
-    client_id = StringProperty(required=True)
-    forum_id = StringProperty(required=True)
+    client_id = StringProperty(required=True, index=True)
+    forum_id = StringProperty(required=True, index=True)
     conversation_id = StringProperty()
     process_datetime = DateTimeProperty(default_now=True)
 
@@ -206,8 +232,8 @@ class ClusterHead(StructuredNode):
     text = StringProperty(required=True)
     similarity_score = FloatProperty(required=True)
     element_count = IntegerProperty()
-    client_id = StringProperty(required=True)
-    forum_id = StringProperty(required=True)
+    client_id = StringProperty(required=True, index=True)
+    forum_id = StringProperty(required=True, index=True)
     conversation_id = StringProperty()
     process_datetime = DateTimeProperty(default_now=True)
     question = Relationship(Question, 'ANSWERS')
@@ -216,8 +242,8 @@ class ClusterHead(StructuredNode):
 class ClusterElement(StructuredNode):
     uid = UniqueIdProperty()
     text = StringProperty(required=True)
-    client_id = StringProperty(required=True)
-    forum_id = StringProperty(required=True)
+    client_id = StringProperty(required=True, index=True)
+    forum_id = StringProperty(required=True, index=True)
     conversation_id = StringProperty()
     process_datetime = DateTimeProperty(default_now=True)
     sentences = Relationship(Sentence, 'DERIVES_FROM')
